/*
    Xunzip for Linux, handles gzip and zip files via zlib.
    Copyright (C) 1998 Tero Koskinen

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
*/

// xunzip.gtk.cpp
// I am sorry about mixture of finnish and english comments :)

#ifdef USE_GTK

#include <unistd.h>
#include <fstream.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <zlib.h>

#include <gtk/gtk.h>
#include <gnome.h>

#include "zipheader.h"
#include "utils.h"
#include "config.h"
#include "tkstring.h"
#include "itoa.h"
#include "debug.h"
#include "linkedlist.h"
#include "xunzip.gtk.h"
#include "zippacket.h"

extern ZipPacket zip;
extern char *fstmp; // for file selection

Widgetit::Widgetit()
{
    filesel=0;
    leveys=390;
    korkeus=250;
    selected_row=-1;
    unzipOneFile=0;
    lowercaseFiles=0;
    ae[0].key='O';
    ae[1].key='P';
    ae[2].key='Q';
    ae[3].key='E';
    ae[4].key='T';
    ae[5].key='V';
    for (int a=0;a<6;a++)
	    ae[a].mod=GDK_CONTROL_MASK;
	    
//    gnome_stock_menu_accel_parse("/gxunzip/Accelerators/");
}

Widgetit::~Widgetit()
{
    TKString s,t;
    pid_t pid;
    char eMessage[FNAME_MAX];
    tyhjennaTaulukko(eMessage,FNAME_MAX);

// just testing how gnome handles menu accel stuff...
#ifdef TK_DEBUG

    guchar key;
    guint8 mod;
    
/*    if (gnome_stock_menu_accel(GNOME_STOCK_MENU_OPEN, &key, &mod))
	    printf("key: %d mod: %d\n",key,mod); */

#endif

    pid=getpid();
    sprintf(eMessage,"/tmp/gxunzip.%d",(int)pid);
    while(!tmpList.atEnd())
    {
	    s=tmpList.getNext();
   	    TK_ASSERT(1,cout << "Removing " << s << "..." << endl;);
	    unlink(s.getStr());
	}
    rmdir(eMessage);
    tmpList.removeAll();
}

Widgetit wi;

void destroy_window (GtkWidget  *,GtkWidget **window)
{
  *window = NULL;
}


// Ohjelman lopetus
void delete_event (GtkWidget *, gpointer *)
{
    zip.close(wi.lista);
    gtk_main_quit ();
}

// purkuikkunan poisto
void delete_unCompress_event(GtkWidget *, GtkWidget *data)
{
  gtk_widget_hide(data);
  gtk_widget_destroy(data);
}

// somebody dropped something?
void dnd_drop(GtkWidget *, GdkEventDropDataAvailable *event)
{
    // TK_ASSERT(1,g_print("dnd_drop\n"););
    // TK_ASSERT(1,g_print("%s\n",event->data_type););
    
    if (event->data && !strcmp(event->data_type,"text/plain"));
    {
  	    if (zip.open((char*)event->data,wi.lista)<0)
		{
		    showError("Cannot open the file ",(char*)event->data);
		}
    }
} 

// user selected a row
void select_row(GtkCList *, gint row, gint , GdkEventButton *)
{
    // TK_ASSERT(1,cout << "select: " << row << endl;);
    wi.selected_row=row;
}

// user unselected a row
void unselect_row(GtkCList *, gint row, gint , GdkEventButton *)
{
    // TK_ASSERT(1,cout << "unselect: " << row << endl;);
    wi.selected_row=-1;
}

void selected_file_toggled (GtkWidget *, gint *)
{
    wi.unzipOneFile=!wi.unzipOneFile;
    // TK_ASSERT(1,cout << "wi.unzipOneFile: " << wi.unzipOneFile << endl;);
}

// void lowercase_selection_toggled (GtkWidget *widget, gint *have_selection)
void lowercase_selection_toggled (GtkWidget *, gint *)
{
    wi.lowercaseFiles=!wi.lowercaseFiles;
    // TK_ASSERT(wi.lowercaseFiles,cout << "Lowercase files" << endl;);
}

gint lowercase_selection_clear (GtkWidget *widget, GdkEventSelection *,gint *have_selection)
{
  *have_selection = FALSE;
  gtk_toggle_button_set_state (GTK_TOGGLE_BUTTON(widget), FALSE);

  return TRUE;
}

void about_cb(GtkWidget *,gpointer *)
{
    const gchar *authors[] = {"Tero Koskinen <tkoskine@jas.pspt.fi>",NULL };
    GtkWidget *about=gnome_about_new ( _("GNOME Xunzip"), VERSION,
                                       _("Copyright (C) 1998 Tero Koskinen"),
                                       authors,NULL,NULL);
    gtk_widget_show (about);
}

void unZip_event (GtkWidget *, GtkWidget *w)
{
    gchar *dir;
    int sfile=-1,options=0;
    // TK_ASSERT(1,cout << "Unzipping to " << gtk_entry_get_text(GTK_ENTRY(wi.entry)) << "..." << endl;);

    if (zip.getType()==3) // ARJ-file
      {
	//	showMessage("Uncompression of ARJ not supported!");
	return;
      }
    dir=gtk_entry_get_text(GTK_ENTRY(wi.entry));
    if (wi.unzipOneFile>0)
    {
	    sfile=wi.selected_row;
	    options|=SELECTED_FILE;
	}
	options|=wi.lowercaseFiles*LOWERCASE_FILENAMES;
    if (zip.unZip(dir,sfile,options)<0)
	{
	    showError(_("Failed uncompress to "),dir);
	}

    gtk_widget_hide(w);
    gtk_widget_destroy(w);
}

void view_event(GtkWidget *,GtkWidget *)
{
    char eMessage[FNAME_MAX];
    TKString s,e;
    pid_t pid;
    tyhjennaTaulukko(eMessage,FNAME_MAX);
	pid=getpid();
	if (wi.selected_row>=0)
	{
	    // QProgressDialog progress(klocale->translate("Viewing file..."),klocale->translate("Cancel"),zip.getFiles(),this,"progress",true);
	    sprintf(eMessage,"/tmp/gxunzip.%d/",(int)pid);
   	    e=zip.getName(wi.selected_row);
   	    s=eMessage;
	    /* TK_ASSERT(1,cout << (s+e) << endl;); */

	    wi.tmpList.add((s+e));
	    if (zip.unZip(s,wi.selected_row,SELECTED_FILE)<0)
	    {
		    showError(_("Cannot view file "),(s+e));
		}
		else
		{
		    s+=e;
		    // e="file:";
		    // s=e+s;
		    // TK_ASSERT(1,cout << "kfm->exec(" << s << ",0L);" << endl;);
		    // kfm->exec(s,0L);
		    create_view(s);
		}
	}
}

void unCompress_event(GtkWidget *,gpointer *)
{
  GSList *buttonGroup;
  static int have_selection=FALSE;
  wi.unzipOneFile=0;
    
  GtkWidget *window;

  GtkWidget *label;
  GtkWidget *okButton;
  GtkWidget *cancelButton;
  GtkWidget *taulukko;
  GtkWidget *selectedFileButton;
  GtkWidget *rButton1;
  GtkWidget *rButton2;

  if (zip.getType()==3) // ARJ-file
    {
      showMessage("Uncompression of ARJ not supported!");
      return;
    }


  window=gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW (window), "Choose directory");
  gtk_container_border_width(GTK_CONTAINER(window),5);
  gtk_widget_set_usize(window,200,120);

  taulukko=gtk_table_new(5,5,TRUE);
  gtk_table_set_row_spacings (GTK_TABLE (taulukko), 1);
  gtk_table_set_col_spacings (GTK_TABLE (taulukko), 1);
  gtk_container_border_width (GTK_CONTAINER (taulukko), 1); 
  gtk_container_add(GTK_CONTAINER(window),taulukko);
	
  // button with OK
  okButton=gtk_button_new_with_label("OK");
  gtk_table_attach_defaults(GTK_TABLE(taulukko),okButton,3,5,0,1);
  gtk_signal_connect(GTK_OBJECT(okButton),"clicked",
		     GTK_SIGNAL_FUNC(unZip_event),window);

  // cancel button
  cancelButton=gtk_button_new_with_label("Cancel");
  gtk_signal_connect(GTK_OBJECT (cancelButton), "clicked",(GtkSignalFunc) delete_unCompress_event,window);
  gtk_table_attach_defaults(GTK_TABLE(taulukko),cancelButton,3,5,1,2);
	
  // text entry
  wi.entry=gtk_entry_new();
  gtk_table_attach_defaults(GTK_TABLE(taulukko),wi.entry,0,3,1,2);
  gtk_entry_set_text(GTK_ENTRY(wi.entry),getenv("HOME"));
	
  // label
  label=gtk_label_new(_("Where to extract?"));
  gtk_table_attach_defaults(GTK_TABLE(taulukko),label,0,3,0,1);
	
  // "lowercase files" button
  selectedFileButton=gtk_check_button_new_with_label(_("Lowercase filenames"));
  gtk_table_attach_defaults(GTK_TABLE(taulukko),selectedFileButton,0,5,4,5);
  gtk_signal_connect (GTK_OBJECT(selectedFileButton), "toggled",
		      GTK_SIGNAL_FUNC (lowercase_selection_toggled), &have_selection);
  gtk_signal_connect (GTK_OBJECT(selectedFileButton), "selection_clear_event",
		      GTK_SIGNAL_FUNC (lowercase_selection_clear), &have_selection);
	
  // radio button (all files/selected file)
  rButton1=gtk_radio_button_new_with_label(NULL,_("Selected file"));
  gtk_table_attach_defaults(GTK_TABLE(taulukko),rButton1,0,5,3,4);
  gtk_signal_connect(GTK_OBJECT(rButton1),"toggled",
		     GTK_SIGNAL_FUNC(selected_file_toggled),NULL);
  buttonGroup=gtk_radio_button_group(GTK_RADIO_BUTTON(rButton1));
  rButton2=gtk_radio_button_new_with_label(buttonGroup,_("All files"));
  /* gtk_signal_connect(GTK_OBJECT(rButton2),"toggled",
     GTK_SIGNAL_FUNC(selected_file_toggled),NULL); */

  gtk_table_attach_defaults(GTK_TABLE(taulukko),rButton2,0,5,2,3);
  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(rButton2),TRUE);
  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(rButton1),FALSE);
  wi.unzipOneFile=0;

    
  //    gtk_signal_connect
  gtk_widget_show(rButton1);
  gtk_widget_show(rButton2);
  gtk_widget_show(selectedFileButton);
  gtk_widget_show(label);
  gtk_widget_show(wi.entry);
  gtk_widget_show(cancelButton);
  gtk_widget_show(okButton);
  gtk_widget_show(taulukko);
  gtk_widget_show(window);
}

void delete_testcrc_event(GtkWidget *,GtkWidget *data)
{
    gtk_widget_hide(data);
    gtk_widget_destroy(data);
}

// testi-ikkuna
void testcrc_event(GtkWidget *,gpointer *)
{
    GtkWidget *window;
    GtkWidget *label;
    GtkWidget *taulukko;
    GtkWidget *lista;
    GtkWidget *swindow;
    GtkWidget *button;

    if (zip.isOpen())
    {

    // paaikkuna
	    window=gtk_window_new(GTK_WINDOW_TOPLEVEL);
	    gtk_window_set_title(GTK_WINDOW(window),"Files with errors");
	    gtk_signal_connect(GTK_OBJECT(window),"destroy",GTK_SIGNAL_FUNC(delete_testcrc_event),window);
	    gtk_container_border_width(GTK_CONTAINER(window),10);
	    gtk_widget_set_usize(window,200,200);
  
    // uusi taulukko
	    taulukko=gtk_table_new(5,3,TRUE);
	    gtk_table_set_row_spacings (GTK_TABLE (taulukko), 1);
	    gtk_table_set_col_spacings (GTK_TABLE (taulukko), 1);
	    gtk_container_border_width (GTK_CONTAINER (taulukko), 1); 
	    gtk_container_add(GTK_CONTAINER(window),taulukko);

    // scrollattava lista (ikkuna, jossa lista)
	    swindow=gtk_scrolled_window_new(NULL,NULL);
	    gtk_widget_set_usize(swindow,100,100);
	    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (swindow),
								  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC); 
	    gtk_table_attach_defaults(GTK_TABLE(taulukko),swindow,0,3,1,4);

    // itse lista nyt
	    lista=gtk_list_new();
	    gtk_list_set_selection_mode (GTK_LIST (lista), GTK_SELECTION_MULTIPLE);
	    gtk_list_set_selection_mode (GTK_LIST (lista), GTK_SELECTION_BROWSE);
	    gtk_container_add(GTK_CONTAINER(swindow),lista);

    // kyltti    
	    label=gtk_label_new("Files which have errors");
	    gtk_table_attach_defaults(GTK_TABLE(taulukko),label,0,3,0,1);
    
    // ja ok-nappi
	    button=gtk_button_new_with_label("OK");
	    gtk_signal_connect(GTK_OBJECT (button), "clicked",(GtkSignalFunc) delete_testcrc_event,window);
	    gtk_table_attach_defaults(GTK_TABLE(taulukko),button,1,2,4,5);
    
	    gtk_widget_show(button);
	    gtk_widget_show(label);
	    gtk_widget_show(lista);
	    gtk_widget_show(swindow);
	    gtk_widget_show(taulukko);
	    gtk_widget_show(window);
	
		zip.testcrc(lista);
	}
}

// hakemiston valinta onnistui hyvaksytysti
void dir_selection_ok (GtkWidget *, GtkFileSelection *fs)
{
    char tiedosto[256];
    for (int a=0;a<256;a++)
	    tiedosto[a]='\0';
    fstmp=gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs));
    strncpy(tiedosto,fstmp,strlen(fstmp));
    filesel_hide();
    if (zip.unZip(tiedosto)<0)
	{
	    showMessage(_("Cannot uncompress the archive"));
	}
}

void dialog_clicked_cb(gchar *string, gpointer )
{
    TKString s;
    if (string)
	{
	    s=string;
	    s+=TKString(" ")+wi.viewed_file;
	    my_system(s);
	    wi.viewed_file=TKString();
	}
}

void create_view(TKString file)
{
    GtkWidget *dialog;
//    GtkWidget *entry;
    wi.viewed_file=file;
    dialog=gnome_request_string_dialog(_("View file with program"),dialog_clicked_cb,NULL);

    gtk_widget_show(dialog);
	
}

void preferences_event(GtkWidget *, gpointer *)
{
    GtkWidget *w;
    
    w=gnome_property_box_new();
    gtk_window_set_title(GTK_WINDOW(w),_("Preferences"));
    gtk_widget_show(w);
}

// piilotetaan hakemiston/tiedoston valintaikkuna
void filesel_hide(void)
{
  if (GTK_WIDGET_VISIBLE(wi.filesel))
	{
	  gtk_widget_hide(wi.filesel);
	  gtk_widget_destroy(wi.filesel);
	  wi.filesel=NULL;
	}
}

// tiedosto valittu hyvaksytysti
void file_selection_ok (GtkWidget *, GtkFileSelection *fs)
{
    char tiedosto[256];
    for (int a=0;a<256;a++)
	    tiedosto[a]='\0';
    fstmp=gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs));
    strncpy(tiedosto,fstmp,strlen(fstmp));
    filesel_hide();
    if (tiedosto)
	{
	    if (zip.open(tiedosto,wi.lista)<0)
		{
		    showError("Cannot open the file ",tiedosto);
		}
	}
}

// luodaan/tuhotaan tiedoston valintaikkuna
void open_file_selection(GtkWidget *, gpointer *)
{
    if (wi.filesel==NULL)
	    create_file_selection(0);
    else
	    filesel_hide();
}

/* lista */
GtkWidget *createList(void)
{
    GtkWidget *w;
    char *titles[5]=
    {
	    N_("Name"),
		N_("Date"),
		N_("Ratio"),
		N_("Size"),
		N_("Packed")
	};
	char *tmp[5];
	tmp[0]=_(titles[0]);
	tmp[1]=_(titles[1]);
	tmp[2]=_(titles[2]);
	tmp[3]=_(titles[3]);
	tmp[4]=_(titles[4]);
	
	w=gtk_clist_new_with_titles(5,tmp);
	
	gtk_clist_set_selection_mode(GTK_CLIST(w), GTK_SELECTION_SINGLE);
/*    gtk_clist_set_column_justification(GTK_CLIST(w), 0, GTK_JUSTIFY_CENTER);
    gtk_clist_set_column_justification(GTK_CLIST(w), 2,  GTK_JUSTIFY_CENTER); */
    gtk_clist_set_column_width(GTK_CLIST(w),0,90);
    gtk_clist_set_column_width(GTK_CLIST(w),1,70);
    gtk_clist_set_column_width(GTK_CLIST(w),2,30);
    gtk_clist_set_column_width(GTK_CLIST(w),3,70);
    gtk_clist_set_column_width(GTK_CLIST(w),4,70);
    gtk_clist_column_titles_active(GTK_CLIST(w));
    gtk_clist_set_policy(GTK_CLIST(w),GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    return w;
}


int connect_dnd(void)
{
    char *dropTypes[] = { "text/plain","url:ALL" };
    gdk_window_dnd_drop_set(GTK_CLIST(wi.lista)->clist_window,TRUE,dropTypes,2,FALSE);
    gtk_signal_connect (GTK_OBJECT (wi.lista), "drop_data_available_event",
					    GTK_SIGNAL_FUNC(dnd_drop), NULL);
    /*
    gtk_widget_dnd_drop_set(wi.lista, 1, dropTypes, 1, 0);
    */

    return TRUE;
}

// luodaan ikkuna ja sihen tarvittavat widgetit
void alustaGTK(char *argv0)
{
/* menu stuff */
GnomeUIInfo filemenu[] = {
    (GnomeUIInfo){GNOME_APP_UI_ITEM,
	 N_("Open"),N_("Open file"),
	 open_file_selection,NULL,NULL,
	 GNOME_APP_PIXMAP_STOCK,GNOME_STOCK_MENU_OPEN,
	 wi.ae[0].key,wi.ae[0].mod,NULL},
	 
	 GNOMEUIINFO_SEPARATOR,
	 (GnomeUIInfo){GNOME_APP_UI_ITEM,
	 N_("Preferences"),N_("Preferences"),
	 preferences_event,NULL,NULL,
	 GNOME_APP_PIXMAP_STOCK,GNOME_STOCK_MENU_PREF,
	 wi.ae[1].key,wi.ae[1].mod,NULL},
	 GNOMEUIINFO_SEPARATOR,
	{GNOME_APP_UI_ITEM,
	 N_("Quit"),N_("Quit"),
	 delete_event,NULL,NULL,
	 GNOME_APP_PIXMAP_STOCK,GNOME_STOCK_MENU_EXIT,
	 wi.ae[2].key,wi.ae[2].mod,NULL},
	 
	GNOMEUIINFO_END
};

GnomeUIInfo actionsmenu[] = {
{GNOME_APP_UI_ITEM,
	 N_("Extract"),NULL,
	 unCompress_event,NULL,NULL,
	 GNOME_APP_PIXMAP_STOCK,GNOME_STOCK_MENU_PROP,
	 wi.ae[3].key,wi.ae[3].mod,NULL},
	{GNOME_APP_UI_ITEM,
	 N_("Test"),NULL,
	 testcrc_event,NULL,NULL,
	 GNOME_APP_PIXMAP_STOCK,GNOME_STOCK_MENU_SEARCH,
	 wi.ae[4].key,wi.ae[4].mod,NULL},
	 	{GNOME_APP_UI_ITEM,
	 N_("View"),NULL,
	 view_event,NULL,NULL,
	 GNOME_APP_PIXMAP_NONE,NULL,
	 wi.ae[5].key,wi.ae[5].mod,NULL},

	GNOMEUIINFO_END
};

GnomeUIInfo helpmenu[] = {
{GNOME_APP_UI_ITEM,
	 N_("About..."),NULL,
	 about_cb,NULL,NULL,
	 GNOME_APP_PIXMAP_STOCK,GNOME_STOCK_MENU_ABOUT,
	 0,(GdkModifierType)0,NULL},
    GNOMEUIINFO_SEPARATOR,
    GNOMEUIINFO_HELP("gxunzip"),
	GNOMEUIINFO_END
};    


GnomeUIInfo mainmenu[] = {
    GNOMEUIINFO_SUBTREE(N_("File"), filemenu),
    GNOMEUIINFO_SUBTREE(N_("Actions"), actionsmenu),
    GNOMEUIINFO_SUBTREE(N_("Help"),helpmenu),
	GNOMEUIINFO_END
};

/* tyokalupalkki (toolbar) */
GnomeUIInfo toolbar[]={
    GNOMEUIINFO_ITEM_STOCK(N_("Open"),N_("Open file..."),
	open_file_selection,GNOME_STOCK_PIXMAP_OPEN),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_ITEM_STOCK(N_("Extract"),N_("Extract file..."),
	unCompress_event,GNOME_STOCK_PIXMAP_PROPERTIES),
	GNOMEUIINFO_ITEM_STOCK(N_("Test"),N_("Check file..."),
	testcrc_event,GNOME_STOCK_PIXMAP_SEARCH),
	GNOMEUIINFO_SEPARATOR,
	GNOMEUIINFO_ITEM_STOCK(N_("Quit"),N_("Quit"),
	delete_event,GNOME_STOCK_PIXMAP_EXIT),
	GNOMEUIINFO_END
};


    char eMessage[100];
/*    char *dropTypes[] = { "text/plain" }; */
//    char *dropTypes[]= { "url:ALL" };

    tyhjennaTaulukko(eMessage,100);
    strcpy(eMessage,"Xunzip ");
    strcat(eMessage,VERSION);

    // paaikkuna
    // wi.window=gtk_window_new(GTK_WINDOW_TOPLEVEL);
	wi.window=gnome_app_new(argv0,"Xunzip");
	gtk_widget_realize(wi.window);
	
    gtk_window_set_title(GTK_WINDOW(wi.window),eMessage);
    gtk_signal_connect(GTK_OBJECT(wi.window),"delete_event",GTK_SIGNAL_FUNC(delete_event),NULL);
    gtk_container_border_width(GTK_CONTAINER(wi.window),1);
    gtk_widget_set_usize(wi.window,wi.leveys,wi.korkeus);
    
    // uusi taulukko
    wi.taulukko=gtk_table_new(6,4,TRUE);
    gtk_table_set_row_spacings (GTK_TABLE (wi.taulukko), 1);
    gtk_table_set_col_spacings (GTK_TABLE (wi.taulukko), 1);
    
    gnome_app_set_contents(GNOME_APP(wi.window), wi.taulukko);
    
    // scrollattava lista (ikkuna, jossa lista)

    // itselista nyt

    wi.lista=createList();
    gtk_signal_connect(GTK_OBJECT(wi.lista),"select_row",GTK_SIGNAL_FUNC(select_row),NULL);
    gtk_signal_connect(GTK_OBJECT(wi.lista),"unselect_row",GTK_SIGNAL_FUNC(unselect_row),NULL);
    gtk_signal_connect(GTK_OBJECT (wi.lista),"realize",GTK_SIGNAL_FUNC (connect_dnd),NULL);
    gtk_table_attach_defaults(GTK_TABLE(wi.taulukko),wi.lista,0,4,0,6);
    
    /* DnD */
/*    gtk_widget_realize(wi.taulukko); */
/*    gtk_widget_realize(wi.lista);

    gtk_signal_connect (GTK_OBJECT (wi.lista), "drop_data_available_event",
					    GTK_SIGNAL_FUNC(dnd_drop), NULL); 
    gtk_widget_dnd_drop_set(wi.lista, 1, dropTypes, 1, 0); */
    // menujen luonti

    gnome_app_create_menus(GNOME_APP(wi.window),mainmenu);
    gtk_menu_item_right_justify(GTK_MENU_ITEM(mainmenu[2].widget));    
    // toolbarin luonti
    gnome_app_create_toolbar(GNOME_APP(wi.window), toolbar);
    
    gtk_widget_show(wi.lista);

    gtk_widget_show(wi.window);
}

// avataan ikkuna josta voi valita tiedoston/hakemiston jotakin toimintoa varten
void create_file_selection (int type)
{
    if (type==0)
	    wi.filesel = gtk_file_selection_new ("File selection");
	else
	    wi.filesel = gtk_file_selection_new ("Directory selection");

    gtk_window_position (GTK_WINDOW (wi.filesel), GTK_WIN_POS_MOUSE);
    gtk_signal_connect (GTK_OBJECT (wi.filesel), "destroy",
					  (GtkSignalFunc) destroy_window,
					  &wi.filesel);
      
    if (type==0)
	    gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (wi.filesel)->ok_button),
					  "clicked", (GtkSignalFunc) file_selection_ok,wi.filesel);
    else
	    gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(wi.filesel)->ok_button),
				      "clicked",(GtkSignalFunc)dir_selection_ok,wi.filesel);
    gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION (wi.filesel)->cancel_button),
							 "clicked", (GtkSignalFunc) gtk_widget_destroy,
							 GTK_OBJECT (wi.filesel));
    gtk_widget_show(wi.filesel);
}

#endif /* USE_GTK */
